==> matrix_sdk-0.4.2.tar.gz: lib/src/storables.dart <==
// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/src/event/event.dart';
import 'package:matrix_sdk/src/event/room/message_event.dart';
import 'package:matrix_sdk/src/event/room/state/state_event.dart';
import 'package:matrix_sdk/src/identifier.dart';
import 'package:matrix_sdk/src/store.dart';
import 'package:meta/meta.dart';
/// [Identifiable] objects that can possibly be stored.
abstract class Storables<I extends Id, T extends Identifiable<I>> {
  final Map<I, T> _memory = Map();

  final Store store;

  Storables({this.store});

  @protected
  Future<void> saveAll(Iterable<T> value);

  @protected
  Future<T> retrieve(I id);

  @protected
  Stream<T> retrieveAll();

  @protected
  Stream<T> retrieveUpTo(int count, {@required T from});

  @protected
  Stream<T> retrieveRemote(int count, {@required T from}) { return null; }

  @protected
  Future<void> removeInStore(I id) { return null; }

  @protected
  Iterable<T> sort(Iterable<T> items) { return items; }

  Future<void> remove(I id) async {
    if (_memory[id] != null) {
      _memory.remove(id);
    }

    await removeInStore(id);
  }

  Future<T> operator [](I id) async {
    // If the room is in memory, return that one
    final fromMemory = _memory[id];
    if (fromMemory != null) {
      return fromMemory;
    }

    // Look in the store
    return await retrieve(id);
  }

  Future<void> addAll(Iterable<T> values) async {

    final nonNullValues = values.where((value) => value != null);

    // Add to memory
    for (final value in nonNullValues) {
      _memory[value.id] = value;
    }

    await saveAll(nonNullValues);
  }

  /// Return all objects from memory and store.
  Stream<T> all() async* {
    for (var item in _memory.values) {
      yield item;
    }

    // Load all events to memory
    await for (var item in retrieveAll()) {
      bool shouldYield = false;

      // If the event was not already in memory, yield it
      _memory.putIfAbsent(item.id, () {
        shouldYield = true;
        return item;
      });

      if (shouldYield) {
        yield item;
      }
    }
  }

  /// Returns up to [count]. First from memory, then from the store.
  /// If not enough items have been yielded, looks up more items with
  /// [retrieveRemote].
  Stream<T> upTo(int count) async* {
    var i = 0;
    var item;

    for (item in sort(_memory.values)) {
      yield item;
      i++;

      if (i >= count) {
        return;
      }
    }

    await for (item in retrieveUpTo(count - i, from: item)) {
      bool shouldYield = false;

      // If the event was not already in memory, yield it
      _memory.putIfAbsent(item.id, () {
        shouldYield = true;
        return item;
      });

      if (shouldYield) {
        yield item;
        i++;
      }

      if (i >= count) {
        return;
      }
    }

    await for (item in retrieveRemote(count - i, from: item)) {
      bool shouldYield = false;

      // If the event was not already in memory, yield it
      _memory.putIfAbsent(item.id, () {
        shouldYield = true;
        return item;
      });

      if (shouldYield) {
        yield item;
        i++;
      }

      if (i >= count) {
        return;
      }
    }
  }
}

==> matrix_sdk-0.7.0.tar.gz: lib/src/storables.dart <==
// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';

import 'package:matrix_sdk/src/identifier.dart';
import 'package:matrix_sdk/src/store.dart';
import 'package:meta/meta.dart';

/// [Identifiable] objects that can possibly be stored.
abstract class Storables<I extends Id, T extends Identifiable<I>> {
  final Map<I, T> _memory = Map();

  final Store store;

  Storables({this.store});

  @protected
  Future<void> saveAll(Iterable<T> value);

  @protected
  Future<T> retrieve(I id);

  @protected
  Stream<T> retrieveAll();

  @protected
  Stream<T> retrieveUpTo(int count, {@required T from});

  @protected
  Stream<T> retrieveRemote(int count, {@required T from}) { return null; }

  @protected
  Future<void> removeInStore(I id) { return null; }

  @protected
  Iterable<T> sort(Iterable<T> items) { return items; }

  @protected
  void removeInMemory(I id) {
    _memory.remove(id);
  }

  Future<void> remove(I id) async {
    removeInMemory(id);
    await removeInStore(id);
  }

  FutureOr<T> operator [](I id) {
    // If the room is in memory, return that one
    final fromMemory = _memory[id];
    if (fromMemory != null) {
      return fromMemory;
    }

    // Look in the store
    return retrieve(id);
  }

  Future<void> addAll(Iterable<T> values) async {

    final nonNullValues = values.where((value) => value != null);

    // Add to memory
    for (final value in nonNullValues) {
      _memory[value.id] = value;
    }

    await saveAll(nonNullValues);
  }

  /// Return all objects from memory and store.
  Stream<T> all() async* {
    for (var item in sort(_memory.values)) {
      yield item;
    }

    // Load all events to memory
    await for (var item in retrieveAll()) {
      bool shouldYield = false;

      // If the event was not already in memory, yield it
      _memory.putIfAbsent(item.id, () {
        shouldYield = true;
        return item;
      });

      if (shouldYield) {
        yield item;
      }
    }
  }

  /// Returns up to [count]. First from memory, then from the store.
  /// If not enough items have been yielded, looks up more items with
  /// [retrieveRemote].
  Stream<T> upTo(int count) async* {
    var i = 0;
    var item;

    for (item in sort(_memory.values)) {
      yield item;
      i++;

      if (i >= count) {
        return;
      }
    }

    await for (item in retrieveUpTo(count - i, from: item)) {
      bool shouldYield = false;

      // If the event was not already in memory, yield it
      _memory.putIfAbsent(item.id, () {
        shouldYield = true;
        return item;
      });

      if (shouldYield) {
        yield item;
        i++;
      }

      if (i >= count) {
        return;
      }
    }

    await for (item in retrieveRemote(count - i, from: item)) {
      bool shouldYield = false;

      // If the event was not already in memory, yield it
      _memory.putIfAbsent(item.id, () {
        shouldYield = true;
        return item;
      });

      if (shouldYield) {
        yield item;
        i++;
      }

      if (i >= count) {
        return;
      }
    }
  }
}
